######################################
# Figures for RDPerm (Canay & Kamat) #
######################################

library(readstata13)
library(rdrobust)
rm(list = ls())

# read in
setwd("/Users/vishal/Documents/RESEARCH/Working Paper/RDD_Perm/Application_round2/figures_R")

# data to use
tabletwo <- read.dta13("table_two.dta")

##----Figure for Dem. vote share in t-1-----------------------------------------

dfdemshareprevleft  <- subset(tabletwo, use==1 & difdemshare<0 & difdemshare>=-0.0166)
dfdemshareprevright <- subset(tabletwo, use==1 & difdemshare<=0.0145 & difdemshare>=0)

pdf("rdplot_dist_demshareprev.pdf")
op <- par(cex=0.64) 
par(oma=c(0,1.5,0,0)) 
par(mar=c(5.1,4.4,1,1))
hist(dfdemshareprevleft$demshareprev, freq = FALSE, border = "white", breaks=seq(0,1,by=0.05),   
     col="orange", xlab="Democrat vote share t-1", main = NULL, cex.axis = 1.6, cex.lab = 1.9)
hist(dfdemshareprevright$demshareprev, freq = FALSE, border = "white", breaks=seq(0,1,by=0.05), 
     col=rgb(0,0,1,0.5), add=T)
legend('topleft', legend=c("Left of threshold", "Right of threshold", "Overlap"),
       fill=c("orange", rgb(0,0,1,0.5), rgb(0.139,0,0.139,0.6)), cex=1.5,
       density = c(NA, NA), ncol=1, border=c("orange", rgb(0,0,1,0.5), rgb(0.139,0,0.139,0.6)),bty="n")
box()
dev.off()

##----Figure for Dem. win in t-1-----------------------------------------

dfdemwinprevleft  <- subset(tabletwo, use==1 & difdemshare<0 & difdemshare>=-0.0179)
dfdemwinprevright <- subset(tabletwo, use==1 & difdemshare<=0.0159 & difdemshare>=0)

pdf("rdplot_dist_demwinprev.pdf")
op <- par(cex=0.64) 
par(oma=c(0,1.5,0,0)) 
par(mar=c(5.1,4.4,1,1))
hist(dfdemwinprevleft$demwinprev, freq = FALSE, border = "white", breaks=seq(0,1,by=0.05),   
     col="orange", xlab="Democrat win t-1", main = NULL, cex.axis = 1.6, cex.lab = 1.9)
hist(dfdemwinprevright$demwinprev,  freq = FALSE, border = "white", breaks=seq(0,1,by=0.05), 
     col=rgb(0,0,1,0.5), add=T)
legend('topleft', legend=c("Left of threshold", "Right of threshold", "Overlap"),
       fill=c("orange", rgb(0,0,1,0.5), rgb(0.139,0,0.139,0.6)), cex=1.5,
       density = c(NA, NA), ncol=1, border=c("orange", rgb(0,0,1,0.5), rgb(0.139,0,0.139,0.6)),bty="n")
box()
dev.off()

##----Figure for Dem. political exp. in t-----------------------------------------

dfleft  <- subset(tabletwo, use==1 & difdemshare<0 & difdemshare>=-0.0207)
dfright <- subset(tabletwo, use==1 & difdemshare<=0.0181 & difdemshare>=0)

pdf("rdplot_dist_demofficeexp.pdf")
op <- par(cex=0.64) 
par(oma=c(0,1.5,0,0)) 
par(mar=c(5.1,4.4,1,1))
hist(dfleft$demofficeexp, freq = FALSE, border = "white", breaks=seq(0,20,by=1),   
     col="orange", xlab="Democrat political experience t", main = NULL, cex.axis = 1.6, cex.lab = 1.9)
hist(dfright$demofficeexp,  freq = FALSE, border = "white", breaks=seq(0,20,by=1), 
     col=rgb(0,0,1,0.5), add=T)
legend('topleft', legend=c("Left of threshold", "Right of threshold", "Overlap"),
       fill=c("orange", rgb(0,0,1,0.5), rgb(0.139,0,0.139,0.6)), cex=1.5,
       density = c(NA, NA), ncol=1, border=c("orange", rgb(0,0,1,0.5), rgb(0.139,0,0.139,0.6)),bty="n")
box()
dev.off()

##----Figure for Opp. political exp. in t-----------------------------------------

dfleft  <- subset(tabletwo, use==1 & difdemshare<0 & difdemshare>=-0.0203)
dfright <- subset(tabletwo, use==1 & difdemshare<=0.0181 & difdemshare>=0)

pdf("rdplot_dist_othofficeexp.pdf")
op <- par(cex=0.64) 
par(oma=c(0,1.5,0,0)) 
par(mar=c(5.1,4.4,1,1))
hist(dfleft$othofficeexp, freq = FALSE, border = "white", breaks=seq(0,20,by=1), ylim = c(0,0.8),  
     col="orange", xlab="Opposition political experience t", main = NULL, cex.axis = 1.6, cex.lab = 1.9)
hist(dfright$othofficeexp,  freq = FALSE, border = "white", breaks=seq(0,20,by=1), 
     col=rgb(0,0,1,0.5), add=T)
legend('topleft', legend=c("Left of threshold", "Right of threshold", "Overlap"),
       fill=c("orange", rgb(0,0,1,0.5), rgb(0.139,0,0.139,0.6)), cex=1.5,
       density = c(NA, NA), ncol=1, border=c("orange", rgb(0,0,1,0.5), rgb(0.139,0,0.139,0.6)),bty="n")
box()
dev.off()

##----Figure for Dem. electoral exp. in t-----------------------------------------

dfleft  <- subset(tabletwo, use==1 & difdemshare<0 & difdemshare>=-0.0209)
dfright <- subset(tabletwo, use==1 & difdemshare<=0.0181 & difdemshare>=0)

pdf("rdplot_dist_demelectexp.pdf")
op <- par(cex=0.64) 
par(oma=c(0,1.5,0,0)) 
par(mar=c(5.1,4.4,1,1))
hist(dfleft$demelectexp, freq = FALSE, border = "white", breaks=seq(0,20,by=1),   
     col="orange", xlab="Democrat electoral experience t", main = NULL, cex.axis = 1.6, cex.lab = 1.9)
hist(dfright$demelectexp,  freq = FALSE, border = "white", breaks=seq(0,20,by=1), 
     col=rgb(0,0,1,0.5), add=T)
legend('topleft', legend=c("Left of threshold", "Right of threshold", "Overlap"),
       fill=c("orange", rgb(0,0,1,0.5), rgb(0.139,0,0.139,0.6)), cex=1.5,
       density = c(NA, NA), ncol=1, border=c("orange", rgb(0,0,1,0.5), rgb(0.139,0,0.139,0.6)),bty="n")
box()
dev.off()

##----Figure for Opp. electoral exp. in t-----------------------------------------

dfleft  <- subset(tabletwo, use==1 & difdemshare<0 & difdemshare>=-0.0203)
dfright <- subset(tabletwo, use==1 & difdemshare<=0.0181 & difdemshare>=0)

pdf("rdplot_dist_othelectexp.pdf")
op <- par(cex=0.64) 
par(oma=c(0,1.5,0,0)) 
par(mar=c(5.1,4.4,1,1))
hist(dfleft$othelectexp, freq = FALSE, border = "white", breaks=seq(0,20,by=1), ylim = c(0,0.8),  
     col="orange", xlab="Opposition electoral experience t", main = NULL, cex.axis = 1.6, cex.lab = 1.9)
hist(dfright$othelectexp,  freq = FALSE, border = "white", breaks=seq(0,20,by=1), 
     col=rgb(0,0,1,0.5), add=T)
legend('topleft', legend=c("Left of threshold", "Right of threshold", "Overlap"),
       fill=c("orange", rgb(0,0,1,0.5), rgb(0.139,0,0.139,0.6)), cex=1.5,
       density = c(NA, NA), ncol=1, border=c("orange", rgb(0,0,1,0.5), rgb(0.139,0,0.139,0.6)),bty="n")
box()
dev.off()

##----CDF plot for Dem. vote share in t-1-----------------------------------------

pdf("rdplot_cdf_demshareprev.pdf")
op <- par(cex=0.55) 
par(mar=c(5.1,5.1,1,1))
plot(ecdf(dfdemshareprevleft$demshareprev), verticals=TRUE, col="orange", 
     pch=1, main=NA, cex.axis = 1.6, cex.lab = 1.9, lwd = 2,
     ylab = "Emp. CDF", xlab = "Democrat vote share t-1", do.points=F, col.01line = NULL)
plot(ecdf(dfdemshareprevright$demshareprev), verticals=TRUE, col=rgb(0,0,1,0.5), 
     pch=1, add=T, lwd = 2, do.points=F,col.01line = NULL)
legend('topleft', legend=c("Left of threshold", "Right of threshold"),
       lty=c(1,1),  ncol=1, col=c("orange", rgb(0,0,1,0.5)), bty = "n", cex=1.75, lwd = 2)
dev.off()

##----RDplot for Dem. vote share in t-1-----------------------------------------

dfrdplot <- subset(tabletwo, use==1)
pdf("rdplot_mean_demshareprev.pdf")
op <- par(cex=0.55) 
par(mar=c(5.1,5.1,1,1))
rdplot(dfrdplot$demshareprev, dfrdplot$difdemshare, 
       cex.axis = 1.6, cex.lab = 1.9, lwd = 4, title = "", col.lines = "black",
       x.label = "Difference in vote share t", y.label = "Democrat vote share t-1",
       x.lim = c(-0.25,0.25), y.lim = c(0.3,0.7))
legend('topleft', legend=c("Sample average within bin", "Polynomial fit of order 4"),
       lty=c(1,NA), pch=c(NA,20),
       ncol=1, col=c("black", "black"), cex=1.75, lwd = 1, bty = "n")
dev.off()