function [REJ,pval,q,h] = CCT(w,x,c,bw)

%CCT - RD Mean test from Calonico et al. (2014)
%   OUTPUT
%    - Rejection where 1 indicates rejection of null
%    - pval is the pvalue from test
%    - q is the mean value of obs used from either side of threshold
%    - h is the mean bandwidth used from either side of threshold
%   INPUT
%    - w   is the vector of dependent variable
%    - x   is vector of running variable
%    - c   is threshold value

kerf           = @(z)(abs(z) <= 1).*(1 - abs(z))                           ;%Kernel function - triangular kernel
N              = size(x,1)                                                 ;

d              = w                                                         ;%Indicating outcome for estimating bandwidth for median
if bw == 0
    [h_opt,~]  = rdbwselect_CCT(d(:,1),x,c)                                ;  
end
if bw == 1
    h_opt      = rdbwselect_IK(d(:,1),x,c)                                 ;
end
h              = h_opt                                                     ;
obs_left       = (x < c)                                                   ;%Observations to left of threshold in each sample
X_left         = x(obs_left,:)                                             ;
W_left         = w(obs_left,:)                                             ;
n              = sum(obs_left)                                             ;%Number of observation to left of threshold in each sample
obs_right      = (x >= c)                                                  ;%Observations to right of threshold in each sample
X_right        = x(obs_right,:)                                            ;
W_right        = w(obs_right,:)                                            ;
m              = sum(obs_right)                                            ;%Number of observation to right of threshold in each sample
H_1            = [1 h^(-1)]                                                ;
H_1            = diag(H_1)                                                 ;
H_2            = [1 h^(-1) h^(-2)]                                         ;
H_2            = diag(H_2)                                                 ;
    
X_minus_1      = [ones(n,1) (X_left./h)]                                   ;
X_minus_2      = [ones(n,1) (X_left./h) (X_left./h).^2]                    ;
W_minus        = diag(kerf(X_left./h)./h)                                  ;
Pi_minus_1     = ((X_minus_1.')*W_minus*X_minus_1)./N                      ;
Pi_minus_2     = ((X_minus_2.')*W_minus*X_minus_2)./N                      ;
%S_minus_1      = X_left./h                                                ;
S_minus_2      = (X_left./h).^2                                            ;
est_minus_1    = (H_1*inv(Pi_minus_1)*(X_minus_1.')*W_minus*w(obs_left,:))./N;
est_minus_2    = (H_2*inv(Pi_minus_2)*(X_minus_2.')*W_minus*w(obs_left,:))./N;
mu_minus       = [1 0]*est_minus_1                                         ;
mu_minus_2     = ([0 0 1]*est_minus_2)*2                                   ;
    
[ID,~]          = knnsearch(X_left,X_left,'K',4)                           ; 
W_left_ID       = W_left(ID)                                               ;
epsilon_minus_s = (3/4)*((W_left - mean(W_left_ID(:,2:4),2)).^2)           ;
sigma_minus     = diag(epsilon_minus_s)                                    ;
    
    
Phi_minus_11   = ((X_minus_1.')*W_minus*sigma_minus*W_minus*X_minus_1)./N  ;
Phi_minus_12   = ((X_minus_1.')*W_minus*sigma_minus*W_minus*X_minus_2)./N  ;
Phi_minus_22   = ((X_minus_2.')*W_minus*sigma_minus*W_minus*X_minus_2)./N  ;
Psi_minus_12   = ((X_minus_1.')*W_minus*S_minus_2)./N                      ;
    
B_minus_12     = [1 0]*inv(Pi_minus_1)*Psi_minus_12                        ;
V_minus_1      = ([1 0]*inv(Pi_minus_1)*Phi_minus_11*inv(Pi_minus_1)*([1 0].'))./N;
C_minus_12     = (2*[1 0]*inv(Pi_minus_1)*Phi_minus_12*inv(Pi_minus_2)*([0 0 1].'))./(N*(h^2));
V_minus_2      = (4*[0 0 1]*inv(Pi_minus_2)*Phi_minus_22*inv(Pi_minus_2)*([0 0 1].'))./(N*(h^4));
    
mu_minus_BC    = mu_minus - ((h^2)*(mu_minus_2/2)*B_minus_12)              ;
V_minus_BC     = V_minus_1 - (2*(h^2)*C_minus_12*B_minus_12)/2 + ((h^4)*V_minus_2*(B_minus_12^2))/4;
        
X_plus_1       = [ones(m,1) (X_right./h)]                                  ;
X_plus_2       = [ones(m,1) (X_right./h) (X_right./h).^2]                  ;
W_plus         = diag(kerf(X_right./h)./h)                                 ;
Pi_plus_1      = ((X_plus_1.')*W_plus*X_plus_1)./N                         ;
Pi_plus_2      = ((X_plus_2.')*W_plus*X_plus_2)./N                         ;
%S_plus_1       = X_right./h                                               ;
S_plus_2       = (X_right./h).^2                                           ;
est_plus_1     = (H_1*inv(Pi_plus_1)*(X_plus_1.')*W_plus*w(obs_right,:))./N;
est_plus_2     = (H_2*inv(Pi_plus_2)*(X_plus_2.')*W_plus*w(obs_right,:))./N;
mu_plus        = [1 0]*est_plus_1                                          ;
mu_plus_2      = ([0 0 1]*est_plus_2)*2                                    ;
    
[ID,~]         = knnsearch(X_right,X_right,'K',4)                          ;
W_right_ID     = W_right(ID)                                               ;
epsilon_plus_s = (3/4)*((W_right - mean(W_right_ID(:,2:4),2)).^2)          ;
sigma_plus     = diag(epsilon_plus_s)                                      ;
    
Phi_plus_11    = ((X_plus_1.')*W_plus*sigma_plus*W_plus*X_plus_1)./N       ;
Phi_plus_12    = ((X_plus_1.')*W_plus*sigma_plus*W_plus*X_plus_2)./N       ;
Phi_plus_22    = ((X_plus_2.')*W_plus*sigma_plus*W_plus*X_plus_2)./N       ;
Psi_plus_12    = ((X_plus_1.')*W_plus*S_plus_2)/N                          ;
    
B_plus_12      = [1 0]*inv(Pi_plus_1)*Psi_plus_12                          ;
V_plus_1       = ([1 0]*inv(Pi_plus_1)*Phi_plus_11*inv(Pi_plus_1)*([1 0].'))./N;
C_plus_12      = (2*[1 0]*inv(Pi_plus_1)*Phi_plus_12*inv(Pi_plus_2)*([0 0 1].'))./(N*(h^2));
V_plus_2       = (4*[0 0 1]*inv(Pi_plus_2)*Phi_plus_22*inv(Pi_plus_2)*([0 0 1].'))./(N*(h^4));
    
mu_plus_BC     = mu_plus - ((h^2)*(mu_plus_2/2)*B_plus_12)                 ;
V_plus_BC      = V_plus_1 - (2*(h^2)*C_plus_12*B_plus_12)/2 + ((h^4)*V_plus_2*(B_plus_12^2))/4;
    
T_stat_BC     = abs(mu_plus_BC - mu_minus_BC)/(sqrt(V_plus_BC + V_minus_BC));

%--------------------------------------------------------------------------
%---------------------------Results----------------------------------------
%--------------------------------------------------------------------------


REJ               = mean((T_stat_BC > 1.96))   ;                            
h                 = h;
n                 = sum((x < c & x >= -h));
m                 = sum((x >= c & x <= h));
q                 = (n+m)/2;
pval              = 2*(1 - normcdf(T_stat_BC));

end
                                             
