function [REJ,h,q,SZ_stat] = SZ(w,x,c,c_h,bw)

%SZ - RD Distribution test from Shen and Zhang(2016)
%   OUTPUT
%    - Rejection where 1 indicates rejection of null
%    - h is bandwidth value used
%    - q is the mean value of obs used from one side of threshold
%    - SZ_stat is the test statistic value obtained
%   INPUT
%    - w   is the vector of dependent variable
%    - x   is vector of running variable
%    - c   is threshold value
%    - c_h is undersmoothing parameter for bandwidth
   
   kerf       = @(z)(abs(z) <= 1).*(1 - abs(z))                            ;
   N          = size(x,1)                                                  ;
   d          = zeros(N,1)                                                 ;

%--Bandwidth selection/Rule of thumb 
                                                  
   d(:,1)     = w                                                          ;%Indicating outcome for estimating bandwidth for median
   if bw == 0
     [h_pilot,~]  = rdbwselect_CCT(d(:,1),x,c)                             ;  
   end
   if bw == 1
     h_pilot      = rdbwselect_IK(d(:,1),x,c)                              ;
   end
   h          = h_pilot                                                    ;
   obs_left   = (x < c & x >= -h)                                          ;%Observations to left of threshold in each sample
   n          = sum(obs_left)                                              ;%Number of observation to left of threshold in each sample
   obs_right  = (x >= c & x <= h)                                          ;%Observations to right of threshold in each sample
   m          = sum(obs_right)                                             ;%Number of observation to right of threshold in each sample
   w_left     = w(obs_left,:)                                              ;
   w_right    = w(obs_right,:)                                             ;
   x_left     = x(obs_left,:)                                              ;
   x_right    = x(obs_right,:)                                             ;

   X_left     = [ones(n,1) x_left]                                         ;                                
   A_left     = ((X_left.')*diag(kerf(x_left/h))*X_left)                   ;
   W_left_1   = repmat(w',n,1)                                             ;
   W_left_2   = repmat(w_left,1,N)                                         ;
   m_minus    = [1 0]*(A_left\((X_left.')*diag(kerf(x_left/h))*...
                (W_left_2 <= W_left_1)))                                   ;
   
   X_right    = [ones(m,1) x_right]                                        ;
   A_right    = ((X_right.')*diag(kerf(x_right/h))*X_right)                ;
   W_right_1  = repmat(w',m,1)                                             ;
   W_right_2  = repmat(w_right,1,N)                                        ;
   m_plus     = [1 0]*(A_right\((X_right.')*diag(kerf(x_right/h))*...
                (W_right_2 <= W_right_1)))                                 ;
   
   KS         = abs(m_plus - m_minus)                                      ;
   KS_max     = max(KS)                                                    ;
   w_max      = w(KS == KS_max)                                            ; 
   w_tilde    = min(w_max)                                                 ;
   d(:,1)     = (w <= w_tilde)                                             ;%Indicating outcome for estimating bandwidth for median
   if bw == 0
      [h_opt,~]   = rdbwselect_CCT(d(:,1),x,c)                             ;
   end
   if bw == 1
      h_opt       = rdbwselect_IK(d(:,1),x,c)                              ; 
   end
   h          = h_opt*(N^((1/5)-(1/c_h)))                                  ;       
   if isnan(h) ==1
   w_tilde    = median(w)                                                  ;
   d(:,1)     = (w <= w_tilde)                                             ;%Indicating outcome for estimating bandwidth for median
   if bw == 0
     [h_opt,~]   = rdbwselect_CCT(d(:,1),x,c)                              ;  
   end
   if bw == 1
     h_opt       = rdbwselect_IK(d(:,1),x,c)                               ;   
   end
   h          = h_opt*(N^((1/5)-(1/c_h)))                                  ;
   if isnan(h) ==1
      h       = h_pilot*(N^((1/5)-(1/c_h)))                                ;
   end
   end
   
%--Implementing testing with rule of thumb bandwidth-----------------------
   
   obs_left   = (x < c & x >= -h)                                          ;%Observations to left of threshold in each sample
   n          = sum(obs_left)                                              ;%Number of observation to left of threshold in each sample
   obs_right  = (x >= c & x <= h)                                          ;%Observations to right of threshold in each sample
   m          = sum(obs_right)                                             ;%Number of observation to right of threshold in each sample
   f          = sum(kerf(x/h))./(N*h)                                      ;%Density estimate at threshold value using both sides in each sample
   w_left     = w(obs_left,:)                                              ;
   w_right    = w(obs_right,:)                                             ;
   w_eff      = [w_left ; w_right]                                         ;
   N_eff      = size(w_eff,1)                                              ;
   x_left     = x(obs_left,:)                                              ;
   x_right    = x(obs_right,:)                                             ;

   X_left     = [ones(n,1) x_left]                                         ;                                
   A_left     = ((X_left.')*diag(kerf(x_left/h))*X_left)                   ;
   W_left_1   = repmat(w_eff',n,1)                                         ;
   W_left_2   = repmat(w(obs_left,:),1,N_eff)                              ;
   m_minus    = [1 0]*(A_left\((X_left.')*diag(kerf(x_left/h))*...
                (W_left_2 <= W_left_1)))                                   ;
            
   X_right    = [ones(m,1) x_right]                                        ;
   A_right    = ((X_right.')*diag(kerf(x_right/h))*X_right)                ;
   W_right_1  = repmat(w_eff',m,1)                                         ;
   W_right_2  = repmat(w(obs_right,:),1,N_eff)                             ;
   m_plus     = [1 0]*(A_right\((X_right.')*diag(kerf(x_right/h))*...
                (W_right_2 <= W_right_1)))                                 ;
   
   KS_stat    = max(abs(m_plus - m_minus))                                 ;%KS TS in each sample
   SZ_stat    = sqrt(5/24)*sqrt(N*h*f./2).*KS_stat                         ;%Shen & Zhang (SZ) test statistic value in each sample
   REJ        = (SZ_stat > 1.3581)                                         ;
   q          = (n + m)/2                                                  ;
   
end

