clear all;
clc;

%%
%--------------------------------------------------------------------------
%---------------------Import data from spreadsheet-------------------------
%--------------------------------------------------------------------------

% Script for importing data from the following spreadsheet:
%
%    Workbook: /Users/vishal/Documents/MATLAB/Working
%    papers/RDD_Perm/Application/Lee_2008/Data Excel/table_two.xls
%    Worksheet: table_two
%
% To extend the code to different selected data or a different spreadsheet,
% generate a function instead of a script.

% Auto-generated by MATLAB on 2015/05/04 15:38:51

% Import the data
[~, ~, raw] = xlsread('/Users/vishal/Documents/RESEARCH/Working Paper/RDD_Perm/Application_round2/table_two.xls','table_two');
raw = raw(2:end,:);

% Replace non-numeric cells with 10000.0
R = cellfun(@(x) ~isnumeric(x) || isnan(x),raw); % Find non-numeric cells
raw(R) = {10000.0}; % Replace non-numeric cells

% Create output variable
data = cell2mat(raw);

% Allocate imported array to column variable names
statedisdec = data(:,1);
demsharenext = data(:,2);
demshareprev = data(:,3);
use = data(:,4);
demwinprev = data(:,5);
difdemshare = data(:,6);
difdemshare2 = data(:,7);
difdemshare3 = data(:,8);
difdemshare4 = data(:,9);
demofficeexp = data(:,10);
demelectexp = data(:,11);
othofficeexp = data(:,12);
othelectexp = data(:,13);
right = data(:,14);
rdifdemshare = data(:,15);
rdifdemshare2 = data(:,16);
rdifdemshare3 = data(:,17);
rdifdemshare4 = data(:,18);
difdemsharenext = data(:,19);

% Clear temporary variables
clearvars data raw R columnIndices;

%%
%--------------------------------------------------------------------------
%------------------------Analysis------------------------------------------
%--------------------------------------------------------------------------


P                = 999                                                     ;
alpha            = 0.05                                                    ;
pval             = zeros(8,1)                                              ;
pval_8           = zeros(8,1)                                              ;
pval_12          = zeros(8,1)                                              ;
q_eff_8          = zeros(7,1)                                              ;
q_eff_12         = zeros(7,1)                                              ;
q_eff            = zeros(7,1)                                              ;
hl               = zeros(7,1)                                              ;
hr               = zeros(7,1)                                              ;
pval_CCT         = zeros(6,1)                                              ;
q_CCT            = zeros(6,1)                                              ;
h_CCT            = zeros(6,1)                                              ;
rej_SZ           = zeros(6,1)                                              ;

   
%-------Testing exogeneity of `demshareprev':Col3 of Table 1 Lee(2008)-----

z1     = difdemshare(demshareprev ~= 10000 & use == 1)                     ;
w1     = demshareprev(demshareprev ~= 10000 & use == 1)                    ;

[~,~,pval(1,1),q_eff(1,1),hl(1,1),hr(1,1)]    = rdperm(w1, z1, 0, alpha,...
                                                       1,1, P,0)      ; 
[~,~,pval_8(1,1),q_eff_8(1,1),~,~]            = rdperm(w1, z1, 0, alpha,...
                                                       1,0.8, P,0)        ;  
[~,~,pval_12(1,1),q_eff_12(1,1),~,~]          = rdperm(w1, z1, 0, alpha,...
                                                       1,1.2, P,0)        ;                                                                                   
[~,pval_CCT(1,1),q_CCT(1,1),h_CCT(1,1)]       = CCT(w1,z1,0,0)             ;
[rej_SZ(1,1),~,q_SZ,SZ_stat]                  = SZ(w1,z1,0,4.5,0)          ;
seq                                           = [1:1:200];
pval_SZ                                       = 2*sum(((-1).^(seq-1)).*exp( -2*seq.^2*SZ_stat^2));

%-------Testing exogeneity of `demwinprev':Col4 of Table 1 Lee(2008)-----

z2     = difdemshare(demwinprev ~= 10000 & use == 1)                       ;
w2     = demwinprev(demwinprev ~= 10000 & use == 1)                        ;
[~,~,pval(2,1),q_eff(2,1),hl(2,1),hr(2,1)]    = rdperm(w2, z2, 0, alpha,...
                                                       1,1, P,0)      ;
[~,~,pval_8(2,1),q_eff_8(2,1),~,~]            = rdperm(w2, z2, 0, alpha,...
                                                       1,0.8, P,0)      ;
[~,~,pval_12(2,1),q_eff_12(2,1),~,~]          = rdperm(w2, z2, 0, alpha,...
                                                       1,1.2, P,0)      ;                                                                                           
[~,pval_CCT(2,1),q_CCT(2,1),h_CCT(2,1)]       = CCT(w2,z2,0,0)             ;
[rej_SZ(2,1),~,~,~]                           = SZ(w2,z2,0,4.5,0)          ;

%-------Testing exogeneity of `demofficeexp':Col5 of Table 1 Lee(2008)-----

z3     = difdemshare(demofficeexp ~= 10000 & use == 1)                     ;
w3     = demofficeexp(demofficeexp ~= 10000 & use == 1)                    ;
[~,~,pval(3,1),q_eff(3,1),hl(3,1),hr(3,1)]    = rdperm(w3, z3, 0, alpha,...
                                                       1,1, P,0)      ; 
[~,~,pval_8(3,1),q_eff_8(3,1),~,~]            = rdperm(w3, z3, 0, alpha,...
                                                       1,0.8, P,0)      ;
[~,~,pval_12(3,1),q_eff_12(3,1),~,~]          = rdperm(w3, z3, 0, alpha,...
                                                       1,1.2, P,0)      ;                                                                                                   
[~,pval_CCT(3,1),q_CCT(3,1),h_CCT(3,1)]       = CCT(w3,z3,0,0)             ;
[rej_SZ(3,1),~,~,~]                           = SZ(w3,z3,0,4.5,0)          ;

%-------Testing exogeneity of `othofficeexp':Col6 of Table 1 Lee(2008)-----

z4     = difdemshare(othofficeexp ~= 10000 & use == 1)                     ;
w4     = othofficeexp(othofficeexp ~= 10000 & use == 1)                    ;
[~,~,pval(4,1),q_eff(4,1),hl(4,1),hr(4,1)]    = rdperm(w4, z4, 0, alpha,...
                                                       1,1, P,0)      ;
[~,~,pval_8(4,1),q_eff_8(4,1),~,~]            = rdperm(w4, z4, 0, alpha,...
                                                       1,0.8, P,0)      ;
[~,~,pval_12(4,1),q_eff_12(4,1),~,~]          = rdperm(w4, z4, 0, alpha,...
                                                       1,1.2, P,0)      ;                                                                                                 
[~,pval_CCT(4,1),q_CCT(4,1),h_CCT(4,1)]       = CCT(w4,z4,0,0)             ;
[rej_SZ(4,1),~,~,~]                           = SZ(w4,z4,0,4.5,0)          ;

%-------Testing exogeneity of `demelectexp':Col7 of Table 1 Lee(2008)-----

z5     = difdemshare(demelectexp ~= 10000 & use == 1)                      ;
w5     = demelectexp(demelectexp ~= 10000 & use == 1)                      ;
[~,~,pval(5,1),q_eff(5,1),hl(5,1),hr(5,1)]    = rdperm(w5, z5, 0, alpha,...
                                                       1,1, P,0)      ;
[~,~,pval_8(5,1),q_eff_8(5,1),~,~]            = rdperm(w5, z5, 0, alpha,...
                                                       1,0.8, P,0)      ;
[~,~,pval_12(5,1),q_eff_12(5,1),~,~]          = rdperm(w5, z5, 0, alpha,...
                                                       1,1.2, P,0)      ;                                                                                                 
[~,pval_CCT(5,1),q_CCT(5,1),h_CCT(5,1)]       = CCT(w5,z5,0,0)             ;
[rej_SZ(5,1),~,~,~]                           = SZ(w5,z5,0,4.5,0)          ;

%-------Testing exogeneity of `othelectexp':Col8 of Table 1 Lee(2008)-----

z6     = difdemshare(othelectexp ~= 10000 & use == 1)                      ;
w6     = othelectexp(othelectexp ~= 10000 & use == 1)                      ;
[~,~,pval(6,1),q_eff(6,1),hl(6,1),hr(6,1)]    = rdperm(w6, z6, 0, alpha,...
                                                       1,1, P,0)      ;
[~,~,pval_8(6,1),q_eff_8(6,1),~,~]            = rdperm(w6, z6, 0, alpha,...
                                                       1,0.8, P,0)      ;
[~,~,pval_12(6,1),q_eff_12(7,1),~,~]          = rdperm(w6, z6, 0, alpha,...
                                                       1,1.2, P,0)      ;                                                                                                     
[~,pval_CCT(6,1),q_CCT(6,1),h_CCT(6,1)]       = CCT(w6,z6,0,0)             ;
[rej_SZ(6,1),~,~,~]                           = SZ(w6,z6,0,4.5,0)          ; 

%-------Testing for joint exogeneity--------------------------------------

use_data = (demshareprev ~= 10000 & demwinprev ~= 10000 & demofficeexp ~= 10000 ...
            & othofficeexp ~= 10000 & demelectexp ~= 10000 ...
            & othelectexp ~= 10000 & use == 1)                             ;
zjoint   = difdemshare(use_data);
wjoint   = [demshareprev(use_data) demwinprev(use_data) ...
            demofficeexp(use_data) othofficeexp(use_data) ...
            demelectexp(use_data) othelectexp(use_data)]                   ;      
        
[~,~,pval(7,1),~,~,~]                      = rdperm(wjoint,zjoint,0,alpha,1,1,P,0);
[~,~,pval(8,1),q_eff(7,1),hl(7,1),hr(7,1)] = rdperm(wjoint,zjoint,0,alpha,1,1,P,1);

[~,~,pval_8(7,1),q_eff_8(7,1),~,~]         = rdperm(wjoint,zjoint,0,alpha,1,0.8,P,0);
[~,~,pval_8(8,1),~,~,~]                    = rdperm(wjoint,zjoint,0,alpha,1,0.8,P,1);

[~,~,pval_12(7,1),q_eff_12(7,1),~,~]       = rdperm(wjoint,zjoint,0,alpha,1,1.2,P,0);
[~,~,pval_12(8,1),~,~,~]                   = rdperm(wjoint,zjoint,0,alpha,1,1.2,P,1);


        