function [h,b]=rdbwselect_CCT(w,x,threshold)

% matlab replication of rdbwselect.R/version 0.5 06Jun2014

% INPUTS
% w is the outcome
% x is the scalar forcing variable
% threshold is the value of the threshold for the forcing variable

% OUTPUT	
% h_opt is bandwidth


kerf        = @(z)(abs(z) <= 1).*(1 - abs(z));
c           = threshold;
N           = length(w);% number of observations
x           = x-c;% forcing variable relative to threshold
obs_left    = (x < c);%Observations to left of threshold in each sample
X_l         = x(obs_left,:);
W_l         = w(obs_left,:);
n           = sum(obs_left);%Number of observation to left of threshold in each sample
obs_right   = (x >= c);%Observations to right of threshold in each sample
X_r         = x(obs_right,:);
W_r         = w(obs_right,:);
m           = sum(obs_right);

%----STEP 0

%------Calculating v_n

S_x              = std(x);
IQR_x            = iqr(x);
temp             = [S_x (IQR_x/1.349)];
v_n              = 2.58*min(temp)*N^(-1/5);

%------Calculating c_n
%------Calculating V_33_plus(v_n) + V_33_minus(v_n)

X_plus_3         = [ones(m,1) (X_r) (X_r).^2 (X_r).^3];
W_plus           = diag(kerf(X_r./v_n)./v_n);
Psi_plus_3       = X_plus_3'*W_plus*X_plus_3;
temp             = (X_r < v_n);
sigma_plus       = zeros(m,1);
sigma_plus(temp,:)  = match(W_r(temp,:),X_r(temp,:));
sigma_plus       = diag(sigma_plus);
Phi_plus_33      = (X_plus_3'*W_plus*sigma_plus*W_plus*X_plus_3);
V_plus_33        = ([0 0 0 1]*inv(Psi_plus_3)*Phi_plus_33*inv(Psi_plus_3)*([0 0 0 1].'));

X_minus_3        = [ones(n,1) (X_l) (X_l).^2 (X_l).^3];
W_minus          = diag(kerf(X_l./v_n)./v_n);
Psi_minus_3      = X_minus_3'*W_minus*X_minus_3;
temp             = (X_l > -v_n);
sigma_minus      = zeros(n,1);
sigma_minus(temp,:)  = match(W_l(temp,:),X_l(temp,:));
sigma_minus      = diag(sigma_minus);
Phi_minus_33     = (X_minus_3'*W_minus*sigma_minus*W_minus*X_minus_3);
V_minus_33       = ([0 0 0 1]*inv(Psi_minus_3)*Phi_minus_33*inv(Psi_minus_3)*([0 0 0 1].'));

V_33             = V_plus_33 + V_minus_33;

%------Calculating B_33

B_33             = 1.777778;

%------Calculating global estimates gamma_4_plus + gamma_4_minus

X_plus_4         = [ones(m,1) X_r (X_r).^2 (X_r).^3 (X_r).^4];  
gamma_plus_4     = inv(X_plus_4'*X_plus_4)*X_plus_4'*W_r;

X_minus_4        = [ones(n,1) X_l (X_l).^2 (X_l).^3 (X_l).^4];  
gamma_minus_4    = inv(X_minus_4'*X_minus_4)*X_minus_4'*W_l ;

%------Calculating c_n using above

Num    = 7*N*v_n^(7)*V_33;
Den    = 2*B_33^2*(gamma_plus_4(5,1) - (-1)^(2)*gamma_minus_4(5,1))^2;
C_012  = Num/Den;
c_n    = C_012^(1/9)*N^(-1/9);

%----STEP 1

%------Calculating b_n
%------Calculating V_22_plus(v_n) + V_22_minus(v_n)

X_plus_2         = [ones(m,1) X_r X_r.^2];
W_plus           = diag(kerf(X_r./v_n)./v_n);
Psi_plus_2       = X_plus_2'*W_plus*X_plus_2;
temp             = (X_r < c_n);
sigma_plus       = zeros(m,1);
sigma_plus(temp,:)  = match(W_r(temp,:),X_r(temp,:));
sigma_plus       = diag(sigma_plus);
Phi_plus_22      = (X_plus_2'*W_plus*sigma_plus*W_plus*X_plus_2);
V_plus_22        = 2^2*([0 0 1]*inv(Psi_plus_2)*Phi_plus_22*inv(Psi_plus_2)*([0 0 1].'))./(N*v_n^(4));
V_plus_22        = V_plus_22*(N*v_n^(4))/4;

X_minus_2        = [ones(n,1) X_l X_l.^2];
W_minus          = diag(kerf(X_l./v_n)./v_n);
Psi_minus_2      = X_minus_2'*W_minus*X_minus_2;
temp             = (X_l > -c_n);
sigma_minus      = zeros(n,1);
sigma_minus(temp,:)  = match(W_l(temp,:),X_l(temp,:));
sigma_minus      = diag(sigma_minus);
Phi_minus_22     = (X_minus_2'*W_minus*sigma_minus*W_minus*X_minus_2);
V_minus_22       = 2^2*([0 0 1]*inv(Psi_minus_2)*Phi_minus_22*inv(Psi_minus_2)*([0 0 1].'))./(N*v_n^(4));
V_minus_22       = V_minus_22*(N*v_n^(4))/4;

V_22             = V_plus_22 + V_minus_22;

%------Calculating B_22

B_22             = 1.285714;

%------Calculating V_33_plus(c_n) + V_33_minus(c_n)

X_plus_3         = [ones(m,1) X_r X_r.^2 X_r.^3];  
W_plus           = diag(kerf(X_r./c_n)./c_n);
Psi_plus_3       = X_plus_3'*W_plus*X_plus_3;
Phi_plus_33      = (X_plus_3'*W_plus*sigma_plus*W_plus*X_plus_3);
V_plus_33        = 6^2*([0 0 0 1]*inv(Psi_plus_3)*Phi_plus_33*inv(Psi_plus_3)*([0 0 0 1].'))./(N*c_n^(6));
V_plus_33        = V_plus_33*(N*c_n^(6))/36;

X_minus_3        = [ones(n,1) X_l X_l.^2 X_l.^3]; 
W_minus          = diag(kerf(X_l./c_n)./c_n);
Psi_minus_3      = X_minus_3'*W_minus*X_minus_3;
Phi_minus_33     = (X_minus_3'*W_minus*sigma_minus*W_minus*X_minus_3);
V_minus_33       = 6^2*([0 0 0 1]*inv(Psi_minus_3)*Phi_minus_33*inv(Psi_minus_3)*([0 0 0 1].'))./(N*c_n^(6));
V_minus_33       = V_minus_33*(N*c_n^(6))/36;

V_33_c           = V_plus_33 + V_minus_33;

%------Calculating estimates beta_3_plus(c_n) + beta_3_minus(c_n)
  
X_plus_3         = [ones(m,1) X_r X_r.^2 X_r.^3];
W_plus           = diag(kerf(X_r./c_n)./c_n);
Psi_plus_3       = X_plus_3'*W_plus*X_plus_3;
beta_plus_3      = (inv(Psi_plus_3)*X_plus_3'*W_plus*W_r);
 
X_minus_3        = [ones(n,1) X_l X_l.^2 X_l.^3];
W_minus          = diag(kerf(X_l./c_n)./c_n);
Psi_minus_3      = X_minus_3'*W_minus*X_minus_3;
beta_minus_3     = (inv(Psi_minus_3)*X_minus_3'*W_minus*W_l);

%------Calculating b_n using above

Num     = 5*N*v_n^(5)*V_22;
Den     = 2*B_22^2*(([0 0 0 1]*beta_plus_3 - (-1)^3*[0 0 0 1]*beta_minus_3)^2 + 3*V_33_c);
C_012_2 = Num/Den;
b_n     = C_012_2^(1/7)*N^(-1/7);

%----STEP 2

%------Calculating V_01_plus(v_n) + V_01_minus(v_n)

X_plus_1       = [ones(m,1) X_r];
W_plus         = diag(kerf(X_r./v_n)./v_n);
Psi_plus_1     = X_plus_1'*W_plus*X_plus_1;
temp             = (X_r < v_n);
sigma_plus       = zeros(m,1);
sigma_plus(temp,:)  = match(W_r(temp,:),X_r(temp,:));
sigma_plus       = diag(sigma_plus);
Phi_plus_1     = (X_plus_1'*W_plus*sigma_plus*W_plus*X_plus_1);
V_plus_01      = ([1 0]*inv(Psi_plus_1)*Phi_plus_1*inv(Psi_plus_1)*[1 0]')/N;
V_plus_01      = V_plus_01*N;

X_minus_1      = [ones(n,1) X_l];
W_minus        = diag(kerf(X_l./v_n)./v_n);
Psi_minus_1    = X_minus_1'*W_minus*X_minus_1;
temp             = (X_l > -v_n);
sigma_minus      = zeros(n,1);
sigma_minus(temp,:)  = match(W_l(temp,:),X_l(temp,:));
sigma_minus      = diag(sigma_minus);
Phi_minus_1    = (X_minus_1'*W_minus*sigma_minus*W_minus*X_minus_1);
V_minus_01     = ([1 0]*inv(Psi_minus_1)*Phi_minus_1*inv(Psi_minus_1)*[1 0]')/N;
V_minus_01     = V_minus_01*N;

V_01           = V_plus_01 + V_minus_01 ;

%------Calculating B_01

B_01           = -0.1;

%------Calculating estimates beta_2_plus(b_n) + beta_2_plus(b_n)

X_plus_2       = [ones(m,1) X_r X_r.^2];
W_plus         = diag(kerf(X_r./b_n)./b_n);
Psi_plus_2     = X_plus_2'*W_plus*X_plus_2;
beta_plus_2    = (inv(Psi_plus_2)*X_plus_2'*W_plus*W_r);

X_minus_2      = [ones(n,1) X_l X_l.^2];
W_minus        = diag(kerf(X_l./b_n)./b_n);
Psi_minus_2    = X_minus_2'*W_minus*X_minus_2;
beta_minus_2   = (inv(Psi_minus_2)*X_minus_2'*W_minus*W_l);

%------Calculating V_22_plus(b_n) + V_22_minus(b_n)

X_plus_2       = [ones(m,1) X_r X_r.^2];
W_plus         = diag(kerf(X_r./b_n)./b_n);
temp             = (X_r < b_n);
sigma_plus       = zeros(m,1);
sigma_plus(temp,:)  = match(W_r(temp,:),X_r(temp,:));
sigma_plus       = diag(sigma_plus);
Psi_plus_2     = X_plus_2'*W_plus*X_plus_2;
Phi_plus_22    = (X_plus_2'*W_plus*sigma_plus*W_plus*X_plus_2);
V_plus_22      = ([0 0 1]*inv(Psi_plus_2)*Phi_plus_22*inv(Psi_plus_2)*[0 0 1]');

X_minus_2      = [ones(n,1) X_l X_l.^2]; 
W_minus        = diag(kerf(X_l./b_n)./b_n);
Psi_minus_2    = X_minus_2'*W_minus*X_minus_2;
temp             = (X_l > -b_n);
sigma_minus      = zeros(n,1);
sigma_minus(temp,:)  = match(W_l(temp,:),X_l(temp,:));
sigma_minus      = diag(sigma_minus);
Phi_minus_22   = (X_minus_2'*W_minus*sigma_minus*W_minus*X_minus_2);
V_minus_22     = ([0 0 1]*inv(Psi_minus_2)*Phi_minus_22*inv(Psi_minus_2)*[0 0 1]');

V_22_b         = V_plus_22 + V_minus_22;

%------Calculating h_n using above

Num   = N*v_n*V_01;
Den   = 4*B_01^2*(([0 0 1]*beta_plus_2 - (-1)^(2)*[0 0 1]*beta_minus_2)^2 + 3*V_22_b);
C_01  = Num/Den;
h_opt = C_01^(1/5)*N^(-1/5);
h     = h_opt;
b     = b_n;


end