function [h_opt]=rdbwselect_IK(w,x,threshold)

% matlab replication of rdbwselect.R/version 0.5 06Jun2014

% INPUTS
% w is the outcome
% x is the scalar forcing variable
% threshold is the value of the threshold for the forcing variable

% OUTPUT	
% h_opt is bandwidth


c           = threshold;
N           = length(w);% number of observations
x           = x-c;% forcing variable relative to threshold
obs_left    = (x < c);%Observations to left of threshold in each sample
X_l         = x(obs_left,:);
W_l         = w(obs_left,:);
n           = sum(obs_left);%Number of observation to left of threshold in each sample
obs_right   = (x >= c);%Observations to right of threshold in each sample
X_r         = x(obs_right,:);
W_r         = w(obs_right,:);
m           = sum(obs_right);


%----STEP 0

S_x = std(x);
h_1 = 1.84*S_x*N^(-1/5);

temp           = (X_l >= -h_1);
N_h1_minus     = sum(temp);
sig2_h1_minus  = var(W_l(temp,:));
if sig2_h1_minus == 0
   temp            = (X_l >= -2*h_1);
   sig2_h1_minus   = var(W_l(temp,:));
end

temp           = (X_r <= h_1);
N_h1_plus      = sum(temp);
sig2_h1_plus    = var(W_r(temp,:));
if sig2_h1_plus == 0
   temp            =  (X_r <= 2*h_1);
   sig2_h1_plus    = var(W_r(temp,:));
end

f_h1           = (N_h1_minus + N_h1_plus)/(2*N*h_1);


%----STEP 1

X_l_median     = median(X_l);
X_r_median     = median(X_r);
temp           = (X_l >= X_l_median);
X_temp_1       = X_l(temp,:);
W_temp_1       = W_l(temp,:);
temp           = (X_r <= X_r_median);
X_temp_2       = X_r(temp,:);
W_temp_2       = W_r(temp,:);
X_temp         = [X_temp_1; X_temp_2];
W_temp         = [W_temp_1; W_temp_2];
X_temp         = [(X_temp >= 0) ones(length(X_temp),1) X_temp X_temp.^2 X_temp.^3];
gamma          = inv(X_temp'*X_temp)*(X_temp'*W_temp);

Num = (5*180*(sig2_h1_minus/f_h1));
Den = 2*(1.5*gamma(5,1))^2;
h_2_minus       = (Num/(n*Den))^(1/7);
Num = (5*180*(sig2_h1_plus/f_h1));
Den = 2*(1.5*gamma(5,1))^2;
h_2_plus       = (Num/(m*Den))^(1/7);


temp           = (X_l >= -h_2_minus);
X_temp         = X_l(temp,:);
X_temp         = [ones(sum(temp),1) X_temp X_temp.^2];
W_temp         = W_l(temp,:);
lambda_minus   = inv(X_temp'*X_temp)*(X_temp'*W_temp);
N_h2_minus     = sum(temp);

temp           = (X_r <= h_2_plus);
X_temp         = X_r(temp,:);
X_temp         = [ones(sum(temp),1) X_temp X_temp.^2];
W_temp         = W_r(temp,:);
lambda_plus    = inv(X_temp'*X_temp)*(X_temp'*W_temp);
N_h2_plus      = sum(temp);

mu2_minus_h2m  = 2*lambda_minus(3,1);
mu2_plus_h2p   = 2*lambda_plus(3,1);

r_h            = (sig2_h1_plus*2160)/(N_h2_plus*h_2_plus^4) + ...
                 (sig2_h1_minus*2160)/(N_h2_minus*h_2_minus^4);

h_opt          = ((4.8*(sig2_h1_plus + sig2_h1_minus)/f_h1)/(4*(-0.1*(-mu2_minus_h2m+mu2_plus_h2p)/2)^2 + 0.1^2*r_h))^(1/5)*N^(-1/5);

h              = h_opt;
n              = sum((x < c & x >= -h));
m              = sum((x >= c & x <= h));
q              = (n+m)/2;

end